@@ -20,11 +20,8 @@ import ssl
 import subprocess
 import sys
 import tempfile
-import textwrap
 
 
-HERE = os.path.abspath(os.path.dirname(__file__))
-ROOT = os.path.abspath(os.path.join(HERE, '../..'))
 PYTHON = sys.executable
 TSCRIPT = os.environ['TSCRIPT']
 GET_PIP_URL = "https://bootstrap.pypa.io/get-pip.py"
@@ -36,8 +33,10 @@ DEPS = [
     "mock",
     "nose",
     "pdbpp",
+    "perf",
     "pip",
     "pypiwin32",
+    "pyreadline",
     "setuptools",
     "unittest2",
     "wheel",
@@ -111,7 +110,84 @@ def rm(pattern, directory=False):
                 safe_remove(path)
 
 
+def safe_remove(path):
+    try:
+        os.remove(path)
+    except OSError as err:
+        if err.errno != errno.ENOENT:
+            raise
+    else:
+        print("rm %s" % path)
+
+
+def safe_rmtree(path):
+    def onerror(fun, path, excinfo):
+        exc = excinfo[1]
+        if exc.errno != errno.ENOENT:
+            raise
+
+    existed = os.path.isdir(path)
+    shutil.rmtree(path, onerror=onerror)
+    if existed:
+        print("rmdir -f %s" % path)
+
+
+def recursive_rm(*patterns):
+    """Recursively remove a file or matching a list of patterns."""
+    for root, subdirs, subfiles in os.walk('.'):
+        root = os.path.normpath(root)
+        if root.startswith('.git/'):
+            continue
+        for file in subfiles:
+            for pattern in patterns:
+                if fnmatch.fnmatch(file, pattern):
+                    safe_remove(os.path.join(root, file))
+        for dir in subdirs:
+            for pattern in patterns:
+                if fnmatch.fnmatch(dir, pattern):
+                    safe_rmtree(os.path.join(root, dir))
+
+
+# ===================================================================
+# commands
+# ===================================================================
+
+
+@cmd
+def help():
+    """Print this help"""
+    print('Run "make <target>" where <target> is one of:')
+    for name in sorted(_cmds):
+        print("    %-20s %s" % (name.replace('_', '-'), _cmds[name] or ''))
+
+
+@cmd
+def build():
+    """Build / compile"""
+    sh("%s setup.py build" % PYTHON)
+    # copies compiled *.pyd files in ./psutil directory in order to
+    # allow "import psutil" when using the interactive interpreter
+    # from within this directory.
+    sh("%s setup.py build_ext -i" % PYTHON)
+
+
+@cmd
+def build_exe():
+    """Create exe file."""
+    build()
+    sh("%s setup.py bdist_wininst" % PYTHON)
+
+
+@cmd
+def build_wheel():
+    """Create wheel file."""
+    build()
+    sh("%s setup.py bdist_wheel" % PYTHON)
+
+
+@cmd
 def install_pip():
+    """Install pip"""
     try:
         import pip  # NOQA
     except ImportError:
@@ -139,29 +215,6 @@ def install_pip():
             os.remove(tfile)
 
 
-# ===================================================================
-# commands
-# ===================================================================
-
-
-@cmd
-def help():
-    """Print this help"""
-    print('Run "make <target>" where <target> is one of:')
-    for name in sorted(_cmds):
-        print("    %-20s %s" % (name.replace('_', '-'), _cmds[name] or ''))
-
-
-@cmd
-def build():
-    """Build / compile"""
-    sh("%s setup.py build" % PYTHON)
-    # copies compiled *.pyd files in ./psutil directory in order to
-    # allow "import psutil" when using the interactive interpreter
-    # from within this directory.
-    sh("%s setup.py build_ext -i" % PYTHON)
-
-
 @cmd
 def install():
     """Install in develop / edit mode"""
@@ -173,30 +226,58 @@ def install():
 @cmd
 def uninstall():
     """Uninstall psutil"""
+    try:
+        import psutil
+    except ImportError:
+        clean()
+        return
+    clean()
     install_pip()
     sh("%s -m pip uninstall -y psutil" % PYTHON)
 
+    # Uninstalling psutil on Windows seems to be tricky as we may have
+    # different versions os psutil installed. Also we don't want to be
+    # in the main psutil source dir as "import psutil" will always
+    # succeed so this really removes files from site-packages dir.
+    here = os.getcwd()
+    try:
+        os.chdir('C:\\')
+        while True:
+            try:
+                import psutil  # NOQA
+            except ImportError:
+                clean()
+                return
+            sh("%s -m pip uninstall -y psutil" % PYTHON)
+    finally:
+        os.chdir(here)
+
 
 @cmd
 def clean():
     """Deletes dev files"""
-    rm("*.egg-info", directory=True)
-    rm("*__pycache__", directory=True)
-    rm("build", directory=True)
-    rm("dist", directory=True)
-    rm("htmlcov", directory=True)
-    rm("tmp", directory=True)
-
-    rm("*.bak")
-    rm("*.core")
-    rm("*.orig")
-    rm("*.pyc")
-    rm("*.pyo")
-    rm("*.rej")
-    rm("*.so")
-    rm("*.~")
-    rm(".coverage")
-    rm(".tox")
+    recursive_rm(
+        "$testfn*",
+        "*.bak",
+        "*.core",
+        "*.egg-info",
+        "*.orig",
+        "*.pyc",
+        "*.pyd",
+        "*.pyo",
+        "*.rej",
+        "*.so",
+        "*.~",
+        "*__pycache__",
+        ".coverage",
+        ".tox",
+    )
+    safe_rmtree("build")
+    safe_rmtree(".coverage")
+    safe_rmtree("dist")
+    safe_rmtree("docs/_build")
+    safe_rmtree("htmlcov")
+    safe_rmtree("tmp")
 
 
 @cmd
@@ -273,28 +354,35 @@ def test_by_name():
     except IndexError:
         sys.exit('second arg missing')
     install()
-    sh(textwrap.dedent("""\
-        %s -m nose \
-        psutil\\tests\\test_process.py \
-        psutil\\tests\\test_system.py \
-        psutil\\tests\\test_windows.py \
-        psutil\\tests\\test_misc.py --nocapture -v -m %s""" % (PYTHON, name)))
+    sh("%s -m unittest -v %s" % (PYTHON, name))
 
 
 @cmd
 def test_memleaks():
     """Run memory leaks tests"""
     install()
-    sh("%s test\test_memory_leaks.py" % PYTHON)
+    sh("%s psutil\\tests\\test_memory_leaks.py" % PYTHON)
 
 
 @cmd
 def install_git_hooks():
-    shutil.copy(".git-pre-commit", ".git/hooks/pre-commit")
+    if os.path.isdir('.git'):
+        shutil.copy(".git-pre-commit", ".git\\hooks\\pre-commit")
+
+
+@cmd
+def bench_oneshot():
+    install()
+    sh("%s scripts\\internal\\bench_oneshot.py" % PYTHON)
+
+
+@cmd
+def bench_oneshot_2():
+    install()
+    sh("%s scripts\\internal\\bench_oneshot_2.py" % PYTHON)
 
 
 def main():
-    os.chdir(ROOT)
     try:
         cmd = sys.argv[1].replace('-', '_')
     except IndexError:
